/*
 * Copyright (C) 2014 tkfir.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */

/*
 This class creates a HTML-combobox to be used within an HTML-form. 
 Just send a SQL-query returning two fields from the DB. Whereby the first field
 will be displayed within your HTML-page and the second value is the field beeing 
 returned from the form.
 */
package applications.html;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tkfir
 */
public class HTMLTwinCombo {

    private String query = null;
    private java.sql.Connection connection;
    private Statement stmt;
    private Logger logger;

    public  HTMLTwinCombo(java.sql.Connection c) {
        logger = Logger.getLogger("applications.html");
        setConnection(c);
    }

    /*
     @return this returns the HTML-Code for the query you have provided by setQuery(v)
     @param  name is the name of the HTML-Form-component        
     @param  size is the number of lines displayed before scrolling
     */
    public String getComboBox(String name, int size) {
        StringBuilder combo = new StringBuilder();
        combo.append("<select name=\"");
        combo.append(name).append("\" size=\"").append(Integer.toString(size)).append("\">\r\n");

        if (query != null) {
            try {
                stmt = connection.createStatement();
                ResultSet rs = stmt.executeQuery(getQuery());
                while (rs.next()) {
                    combo.append("\t<option value=\"").append(rs.getString(2)).append("\">");
                    combo.append(rs.getString(1));
                    combo.append("</option>\r\n");
                }
            } catch (SQLException sqlex) {
                logger.log(Level.WARNING, "Exception while reading from DB: {0}", sqlex.getMessage());
            }

        }
        combo.append("</select>\r\n");
        return combo.toString();
    }

    /**
     * @return the query
     */
    public String getQuery() {
        return query;
    }

    /**
     * @param query the query to set
     */
    public void setQuery(String query) {
        this.query = query;
    }

    /**
     * @return the connection
     */
    public Connection getConnection() {
        return connection;
    }

    /**
     * @param connection the connection to set
     */
    public void setConnection(Connection connection) {
        this.connection = connection;
    }

    /**
     * @return the stmt
     */
    public Statement getStmt() {
        return stmt;
    }

    /**
     * @param stmt the stmt to set
     */
    public void setStmt(Statement stmt) {
        this.stmt = stmt;
    }

    /**
     * @return the logger
     */
    public Logger getLogger() {
        return logger;
    }

    /**
     * @param logger the logger to set
     */
    public void setLogger(Logger logger) {
        this.logger = logger;
    }

}


