/*
 * Copyright (C) 2014 tkfir.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package gpl.fredy.metadata;

import java.util.ArrayList;
import javax.swing.table.AbstractTableModel;

/**
 *
 * @author tkfir
 */
public class FindPatternTable extends AbstractTableModel {

    private ArrayList<FindPattern> findPattern = new ArrayList();
    private Object[][] rowData;

    public String[] columnNames = {
        "Database",
        "Table",
        "Column",
        "Pattern"
    };

    @Override
    public String getColumnName(int i) {
        return columnNames[i];
    }

    @Override
    public int getColumnCount() {
        return columnNames.length;
    }

    public static final int DATABASE = 0;
    public static final int TABLE = 1;
    public static final int COLUMN = 2;
    public static final int PATTERN = 3;

    @Override
    public String getValueAt(int row, int col) {
        FindPattern fp = findPattern.get(row);
        if (col == DATABASE) {
            return fp.getDatabase();
        }
        if (col == TABLE) {
            return fp.getTable();
        }
        if (col == COLUMN) {
            return fp.getColumn();
        }
        if (col == PATTERN) {
            return fp.getPattern();
        }

        return null;

    }

    @Override
    public int getRowCount() {
        return findPattern.size();
    }

    /**
     * @return the findPattern
     */
    public ArrayList<FindPattern> getFindPattern() {
        return findPattern;
    }

    /**
     * @param findPattern the findPattern to set
     */
    public void setFindPattern(ArrayList<FindPattern> findPattern) {
        this.findPattern = findPattern;
        int i = 0;

        for (FindPattern fp : findPattern) {
            setValueAt(fp, i, 0);
            /*
             setValueAt(fp.getDatabase(), i, DATABASE);
             setValueAt(fp.getTable(), i, TABLE);
             setValueAt(fp.getColumn(), i, COLUMN);
             setValueAt(fp.getPattern(), i, PATTERN);
             */

            i++;
        }

    }

    @Override
    public boolean isCellEditable(int row, int col) {
        return false;
    }

    @Override
    public void setValueAt(Object value, int row, int col) {
        FindPattern fp = (FindPattern) value;

        /*
         fp.setDatabase("");
         fp.setTable("");
         fp.setColumn("");
         fp.setPattern("");

         if (col == DATABASE) {
         fp.setDatabase((String) value);
         }
         if (col == TABLE) {
         fp.setTable((String) value);
         }
         if (col == COLUMN) {
         fp.setColumn((String) value);
         }
         if (col == PATTERN) {
         fp.setPattern((String) value);
         }

         */
        findPattern.set(row, fp);

        fireTableCellUpdated(row, col);
    }

    public FindPatternTable() {

    }

    public FindPatternTable(ArrayList<FindPattern> findPattern) {
        setFindPattern(findPattern);
    }
}
