package gpl.fredy.sqltools;

/**
 *
 * This software is part of David Good's contribution to the Admin-Framework and
 * under LGPL
 *
 * Admin is a Tool around JDBC-enabled SQL-Databases to do basic jobs for
 * DB-Administrations, as create / delete / alter and query tables it also
 * creates indices and generates simple Java-Code to access DBMS-tables and
 * exports data into various formats
 *
 *
 * Copyright (C) August 9, 2003, Fredy Fischer sql@hulmen.ch Postal: Fredy
 * Fischer Hulmenweg 36 8405 Winterthur Switzerland
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
import java.sql.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.tree.*;
import java.io.*;

import gpl.fredy.share.*;
import gpl.fredy.metadata.*;
import gpl.fredy.ui.*;

public class SqlPickList extends JPanel {

    t_connect con = null;

    /**
     * Get the value of con.
     *
     * @return value of con.
     */
    public t_connect getCon() {
        if ((con == null) || (getSqlCon() == null)) {
            con = new t_connect(getHost(),
                    getUser(),
                    getPassword(),
                    getDatabase());
            if (!con.acceptsConnection()) {
                con = null;
            }
            setSqlCon(con.con);
        }
        return con;
    }

    /**
     * Set the value of con.
     *
     * @param v Value to assign to con.
     */
    public void setCon(t_connect v) {
        this.con = v;
        setUser(con.getUser());
        setHost(con.getHost());
        setPassword(con.getPassword());
        setDatabase(con.getDatabase());
        setSqlCon(con.con);
    }

    String host = "localhost";

    /**
     * Get the value of host.
     *
     * @return Value of host.
     */
    public String getHost() {
        return host;
    }

    /**
     * Set the value of host.
     *
     * @param v Value to assign to host.
     */
    public void setHost(String v) {
        this.host = v;
    }

    String user;

    /**
     * Get the value of user.
     *
     * @return Value of user.
     */
    public String getUser() {
        return user;
    }

    /**
     * Set the value of user.
     *
     * @param v Value to assign to user.
     */
    public void setUser(String v) {
        this.user = v;
    }

    String password;

    /**
     * Get the value of password.
     *
     * @return Value of password.
     */
    public String getPassword() {
        return password;
    }

    /**
     * Set the value of password.
     *
     * @param v Value to assign to password.
     */
    public void setPassword(String v) {
        this.password = v;
    }

    String database;

    /**
     * Get the value of database.
     *
     * @return Value of database.
     */
    public String getDatabase() {
        return database;
    }

    /**
     * Set the value of database.
     *
     * @param v Value to assign to database.
     */
    public void setDatabase(String v) {
        this.database = v;
    }

    String schema;

    /**
     * Get the value of schema.
     *
     * @return Value of schema.
     */
    public String getSchema() {
        return schema;
    }

    /**
     * Set the value of schema.
     *
     * @param v Value to assign to schema.
     */
    public void setSchema(String v) {
        this.schema = v;
    }

    private DatabaseMetaData dmd;
    ResultSet rsTables;
    Vector vTables = new Vector(10, 10);
    JTree jtTables;
    JButton cbPaste;
    Container jPanel1 = this;

    private java.sql.Connection sqlCon = null;

    SqlPickList(String host,
            String user,
            String password,
            String db,
            ActionListener paste,
            String schema) {

        setHost(host);
        setUser(user);
        setPassword(password);
        setDatabase(db);
        setSchema(schema);
        init(paste);
        getCon();
    }

    SqlPickList(t_connect con, ActionListener paste) {
        setCon(con);
        setSchema(con.getSchema());
        init(paste);
    }

    SqlPickList(t_connect con, ActionListener paste, String schema) {
        setCon(con);
        setSchema(schema);
        init(paste);
    }

    SqlPickList(java.sql.Connection con, ActionListener paste, String schema) {
        setSqlCon(con);
        setSchema(schema);
        init(paste);
    }

    SqlPickList(java.sql.Connection con, ActionListener paste) {
        setSqlCon(con);
        try {
            setSchema(con.getSchema());
        } catch (SQLException sqlex) {

        }
        init(paste);
    }
    TreePath selPath;

    public void init(ActionListener paste) {
        popUp();

        try {

            //getCon();
            dmd = getSqlCon().getMetaData();
            rsTables = dmd.getTables(null,
                    schema,
                    null,
                    new String[]{"TABLE", "VIEW"});

            getTableList(rsTables);

            jPanel1.setLayout(new BorderLayout());

            cbPaste = new JButton("Paste");
            cbPaste.addActionListener(paste);
            cbPaste.setActionCommand("");

            jPanel1.add(cbPaste, BorderLayout.SOUTH);

            jtTables = new JTree(vTables);

            jtTables.addMouseListener(new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    selPath = jtTables.getPathForLocation(e.getX(), e.getY());
                    if (selPath != null) {

                        int centerButton = e.BUTTON2;
                        if (MouseInfo.getNumberOfButtons() < 3) {
                            centerButton = 0;
                        }

                        // add popup if once left click
                        if (e.getButton() == centerButton) {
                            pm.show(e.getComponent(), e.getX(), e.getY());
                        } else {
                            cbPaste.setActionCommand(" " + ((PickListItem) ((DefaultMutableTreeNode) (selPath.getLastPathComponent())).getUserObject()).getName() + " ");
                            if (e.getClickCount() == 2) {
                                e.consume();
                                cbPaste.doClick();

                            }
                        }
                    }
                }
            }
            );

            JScrollPane scp = new JScrollPane(jtTables);

            scp.setSize(
                    100, 80);

            jPanel1.add(scp);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void getTableList(ResultSet rs) throws Exception {
        while (rs.next()) {
            vTables.add(new ColumnList(rs.getString(3)));
        }
    }

    public class ColumnList extends Vector implements PickListItem {

        private String tableName;

        public ColumnList(String TableName) throws Exception {
            super(10, 5);
            tableName = TableName;
            ResultSet rs = dmd.getColumns(null, null, tableName, null);
            while (rs.next()) {
                add(new Column(rs.getString(4)));
            }
        }

        public String toString() {
            return tableName;
        }

        public String getName() {
            return tableName;
        }

        public class Column implements PickListItem {

            private String columnName;

            public Column(String columnName) {
                this.columnName = columnName;
            }

            public String toString() {
                return columnName;
            }

            public String getName() {
                return ColumnList.this.tableName + "." + columnName;
            }
        }
    }

    protected void finalize() {
        //con.close();
    }

    /**
     * Getter for property sqlCon.
     *
     * @return Value of property sqlCon.
     */
    public java.sql.Connection getSqlCon() {
        return sqlCon;
    }

    /**
     * Setter for property sqlCon.
     *
     * @param sqlCon New value of property sqlCon.
     */
    public void setSqlCon(java.sql.Connection sqlCon) {
        this.sqlCon = sqlCon;
    }

    JPopupMenu pm;

    private void popUp() {
        pm = new JPopupMenu("Helper");
        pm.add(tablePopUp());
        pm.add(rowPopUp());

        /*
         final JPanel a = this;

         MouseListener mouseListener = new MouseAdapter() {

         // left mouse button
         public void mouseClicked(MouseEvent e) {
         int leftButton = e.BUTTON3;
         if (MouseInfo.getNumberOfButtons() < 3) {
         leftButton = e.BUTTON2;
         }

         // if clicked once  the right , it adds a SQL-Statement to see this table to the clipboard
         if (e.getButton() == leftButton) {
         if (e.getClickCount() == 1) {
         pm.show(a, e.getX(), e.getY());
         }
         }
         }
         };
         this.addMouseListener(mouseListener);
         */
    }

    private JMenu tablePopUp() {
        JMenu popup = new JMenu("Table Helper");

        JMenuItem selStm = new JMenuItem("Create Select Statement");
        selStm.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cbPaste.setActionCommand(createSelectStatement());
                cbPaste.doClick();
            }
        });

        popup.add(selStm);

        return popup;

    }

    private String getSelectedRow() {
        String leaf = null;

        // only if  a Leaf has been selected
        if (((DefaultMutableTreeNode) (selPath.getLastPathComponent())).isLeaf()) {
            leaf = ((PickListItem) ((DefaultMutableTreeNode) (selPath.getLastPathComponent())).getUserObject()).getName();
        }

        return leaf;
    }

    private String getSelectedTable() {
        String t = null;
        if (!((DefaultMutableTreeNode) (selPath.getLastPathComponent())).isLeaf()) {
            t = ((PickListItem) ((DefaultMutableTreeNode) (selPath.getLastPathComponent())).getUserObject()).getName();
        }
        return t;
    }

    // if one uses Spaces in a field- or table name, it has to be surrounded by "
    private String fixSpaces(String v) {
        if (v != null) {
            if (v.trim().contains(" ")) {
                v = "\"" + v + "\"";
            }
        }
        return v;
    }

    private String createSelectStatement() {
        StringBuilder sl = new StringBuilder();
        DefaultMutableTreeNode dmt = (DefaultMutableTreeNode) selPath.getLastPathComponent();
        if (dmt.isLeaf()) {
            dmt = (DefaultMutableTreeNode) dmt.getParent();
        }

        sl.append("select ");

        for (int i = 0; i < dmt.getChildCount(); i++) {
            DefaultMutableTreeNode tn = (DefaultMutableTreeNode) dmt.getChildAt(i);

            // make sure, the last Object does not have a ,
            if (i == dmt.getChildCount() - 1) {
                sl.append(" ").append(fixSpaces(((PickListItem) dmt.getUserObject()).toString())).append(".").append(fixSpaces(((PickListItem) tn.getUserObject()).toString())).append(" \n");
            } else {
                sl.append(" ").append(fixSpaces(((PickListItem) dmt.getUserObject()).toString())).append(".").append(fixSpaces(((PickListItem) tn.getUserObject()).toString())).append(", \n");
            }
        }

        sl.append("from ").append(fixSpaces(((PickListItem) dmt.getUserObject()).toString())).append("\n");

        return sl.toString();
    }

    private StringBuilder sb;

    private JMenu rowPopUp() {

        sb = new StringBuilder();
        JMenu popup = new JMenu("Row Filters");

        JMenu whereMenu = new JMenu("where conditions");
        JMenu andMenu = new JMenu("and conditions");
        JMenuItem orderBy = new JMenuItem("Order by");

        orderBy.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" order by ").append(v);
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }

            }
        });

        JMenuItem whereLike = new JMenuItem("add 'where like'");
        whereLike.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" where ").append(v).append(" like ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }

            }
        });

        JMenuItem whereEqual = new JMenuItem("add 'where ='");
        whereEqual.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" where ").append(v).append(" = ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem whereLess = new JMenuItem("add 'where <'");
        whereLess.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" where ").append(v).append(" < ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem whereBigger = new JMenuItem("add 'where >'");
        whereBigger.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" where ").append(v).append(" > ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem whereBetween = new JMenuItem("add 'where between'");
        whereBetween.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" where ").append(v).append(" between ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem whereIn = new JMenuItem("add 'where in'");
        whereIn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" where ").append(v).append(" in ( ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem whereIsNull = new JMenuItem("add 'where is Null'");
        whereIsNull.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" where ").append(v).append(" IS NULL ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem andLike = new JMenuItem("add 'and like'");
        andLike.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" and ").append(v).append(" like ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }

            }
        });

        JMenuItem andEqual = new JMenuItem("add 'and ='");
        andEqual.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" and ").append(v).append(" = ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem andLess = new JMenuItem("add 'and <'");
        andLess.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" and ").append(v).append(" < ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem andBigger = new JMenuItem("add 'and >'");
        andBigger.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" and ").append(v).append(" > ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem andBetween = new JMenuItem("add 'and between'");
        andBetween.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" and ").append(v).append(" between ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem andIn = new JMenuItem("add 'and in'");
        andIn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" and ").append(v).append(" in ( ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        JMenuItem andIsNull = new JMenuItem("add 'and is Null'");
        andIsNull.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                sb = new StringBuilder();
                String v = null;
                v = fixSpaces(getSelectedRow());
                if (v != null) {
                    sb.append(" and ").append(v).append(" IS NULL ");
                    cbPaste.setActionCommand(sb.toString());
                    cbPaste.doClick();
                }
            }
        });

        whereMenu.add(whereLike);
        whereMenu.add(whereEqual);
        whereMenu.add(whereLess);
        whereMenu.add(whereBigger);
        whereMenu.add(whereBetween);
        whereMenu.add(whereIsNull);

        andMenu.add(andLike);
        andMenu.add(andEqual);
        andMenu.add(andLess);
        andMenu.add(andBigger);
        andMenu.add(andBetween);
        andMenu.add(andIsNull);

        popup.add(whereMenu);
        popup.add(andMenu);
        popup.add(orderBy);

        return popup;
    }

}
